@@ -4,12 +4,13 @@
 from __future__ import absolute_import, unicode_literals
 
 __all__ = ["Star", "Planet", "PlanetarySystem", "LimbDarkening",
-           "QuadraticLimbDarkening", "NonlinearLimbDarkening"]
+           "QuadraticLimbDarkening"]
 
 from .bart import Star, Planet, PlanetarySystem
-from .ldp import LimbDarkening, QuadraticLimbDarkening, NonlinearLimbDarkening
+from .dataset import KeplerDataset
+from . import parameters, kepler
 
-__version__ = "0.0.2"
+__version__ = "0.0.3"
 __author__ = "Dan Foreman-Mackey (danfm@nyu.edu)"
 __copyright__ = "Copyright 2013 Dan Foreman-Mackey"
 __contributors__ = [
@@ -17,7 +18,7 @@ __contributors__ = [
                     "Patrick Cooper @patrickcooper",
                    ]
 
-# Get the git commit hash.
+# Get the git commit hash and save it as the __commit__ variable.
 import os
 import subprocess as sp
 bart_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
@@ -25,7 +26,8 @@ commit_file = os.path.join(bart_dir, "COMMIT")
 p = sp.Popen(["git", "rev-parse", "--short", "HEAD"],
              stdout=sp.PIPE, cwd=bart_dir)
 r = p.wait()
-if False:  # r == 0:
-    __version__ += "-" + p.stdout.read().strip()
+__commit__ = None
+if r == 0:
+    __commit__ = p.stdout.read().strip()
 elif os.path.exists(commit_file):
-    __version__ += "-" + open(commit_file).read().strip()
+    __commit__ = open(commit_file).read().strip()
