@@ -5,9 +5,11 @@ from __future__ import print_function, absolute_import, unicode_literals
 
 __all__ = ["Star", "Planet", "PlanetarySystem"]
 
+import os
 import cPickle as pickle
 
 import numpy as np
+import scipy.optimize as op
 import emcee
 
 try:
@@ -22,8 +24,10 @@ try:
 except ImportError:
     h5py = None
 
-from bart import _bart, mog
+from bart import _bart
+from bart.km import km1d
 from bart.ldp import LimbDarkening
+from bart.results import ResultsProcess
 
 
 _G = 2945.4625385377644
@@ -99,6 +103,22 @@ class Star(Model):
         else:
             self.ldp = ldp
 
+    @property
+    def spec(self):
+        return np.array([self.mass, self.radius, self.flux])
+
+    @spec.setter  # NOQA
+    def spec(self, v):
+        self.mass, self.radius, self.flux = v
+
+    def get_semimajor(self, T):
+        """
+        Get the mass of the host star implied by the semi-major axis of this
+        planet and an input period.
+
+        """
+        return (_G * T * T * self.mass / (4 * np.pi * np.pi)) ** (1. / 3)
+
 
 class Planet(Model):
     """
@@ -111,6 +131,9 @@ class Planet(Model):
     :param a:
         The semi-major axis of the orbit in Solar radii.
 
+    :param mass: (optional)
+        The mass of the planet in Solar masses.
+
     :param t0: (optional)
         The time of a reference pericenter passage.
 
@@ -130,25 +153,38 @@ class Planet(Model):
 
     """
 
-    def __init__(self, r, a, t0=0.0, e=0.0, pomega=0.0, ix=0.0, iy=0.0):
+    def __init__(self, r, a, mass=0.0, t0=0.0, e=0.0, pomega=0.0, ix=0.0,
+                 iy=0.0):
         super(Planet, self).__init__()
 
         self.r = r
         self.a = a
+        self.mass = mass
         self.t0 = t0
         self.e = e
         self.pomega = pomega
         self.ix = ix
         self.iy = iy
 
-    def get_mstar(self, T):
+    @property
+    def spec(self):
+        return np.array([float(self.r), float(self.a), float(self.mass),
+                         float(self.t0), float(self.e), float(self.pomega),
+                         float(self.ix), float(self.iy)])
+
+    @spec.setter  # NOQA
+    def spec(self, v):
+        (self.r, self.a, self.mass, self.t0, self.e, self.pomega, self.ix,
+         self.iy) = v
+
+    def get_mstar(self, P):
         """
         Get the mass of the host star implied by the semi-major axis of this
         planet and an input period.
 
         """
         a = self.a
-        return a * a * a * 4 * np.pi * np.pi / _G / T / T
+        return a * a * a * 4 * np.pi * np.pi / _G / P / P - self.mass
 
     def get_period(self, mstar):
         """
@@ -156,7 +192,7 @@ class Planet(Model):
 
         """
         a = self.a
-        return 2 * np.pi * np.sqrt(a * a * a / _G / mstar)
+        return 2 * np.pi * np.sqrt(a * a * a / _G / (mstar + self.mass))
 
 
 class PlanetarySystem(Model):
@@ -171,18 +207,65 @@ class PlanetarySystem(Model):
 
     """
 
-    def __init__(self, star, iobs=90.0):
+    def __init__(self, star, basepath=".", iobs=90.0, rv0=0.0, pbad=1e-4,
+                 vbad=1e-2):
         super(PlanetarySystem, self).__init__()
 
-        self._data = None
+        self.basepath = basepath
+        try:
+            os.makedirs(basepath)
+        except os.error:
+            pass
+
+        # Data.
+        self.datasets = []
+
+        # Priors.
+        self.priors = []
 
         # The properties of the system as a whole.
         self.star = star
         self.iobs = iobs
+        self.rv0 = rv0
 
         # The planets.
         self.planets = []
 
+        # The background model.
+        self.pbad = pbad
+        self.vbad = vbad
+
+    def add_dataset(self, ds):
+        self.datasets.append(ds)
+
+    def add_prior(self, p):
+        self.priors.append(p)
+
+    @property
+    def spec(self):
+        return np.concatenate([[float(self.iobs), float(self.rv0),
+                                float(self.pbad), float(self.vbad)],
+                                self.star.spec,
+                              np.concatenate([p.spec for p in self.planets])])
+
+    @spec.setter  # NOQA
+    def spec(self, v):
+        nspec = 4
+        self.iobs = v[0]
+        self.rv0 = v[1]
+        self.pbad = v[2]
+        self.vbad = v[3]
+
+        ls = len(self.star.spec)
+        self.star.spec = v[nspec:ls + nspec]
+        lp = len(self.planets[0].spec)
+        for i, p in enumerate(self.planets):
+            p.spec = v[ls + nspec + i * lp:ls + nspec + (i + 1) * lp]
+
+    def results(self, *args, **kwargs):
+        kwargs["basepath"] = kwargs.pop("basepath", self.basepath)
+        return ResultsProcess(*args, **kwargs)
+
     @property
     def nplanets(self):
         """
@@ -210,7 +293,8 @@ class PlanetarySystem(Model):
         """
         v = super(PlanetarySystem, self).vector
         return np.concatenate([v, self.star.vector]
-                              + [p.vector for p in self.planets])
+                              + [p.vector for p in self.planets]
+                              + [d.vector for d in self.datasets])
 
     @vector.setter  # NOQA
     def vector(self, v):
@@ -227,6 +311,9 @@ class PlanetarySystem(Model):
         for p in self.planets:
             p.vector = v[i:i + len(p)]
             i += len(p)
+        for d in self.datasets:
+            d.vector = v[i:i + len(d)]
+            i += len(d)
 
     def sample(self, size, std=1e-5):
         """
@@ -246,6 +333,11 @@ class PlanetarySystem(Model):
             if pv is not None:
                 l.append(pv)
 
+        for d in self.datasets:
+            pv = d.sample(size, std=std)
+            if pv is not None:
+                l.append(pv)
+
         return np.concatenate(l).T
 
     def __call__(self, p):
@@ -284,73 +376,110 @@ class PlanetarySystem(Model):
         Compute the log-prior of the current model.
 
         """
-        lnp = [p.lnprior(self) for p in self.parameters]
+        lnp = [p(self) for p in self.priors]
+        lnp += [p.lnprior(self) for p in self.parameters]
         lnp += [p.lnprior(self.star) for p in self.star.parameters]
         for planet in self.planets:
             lnp += [p.lnprior(planet) for p in planet.parameters]
+        for d in self.datasets:
+            lnp += [p.lnprior(d) for p in d.parameters]
         if np.isinf(np.any(lnp)):
             return -np.inf
         return np.sum(lnp)
 
     def lnlike(self):
         """
-        Compute the log-likelihood of the current model.
+        Compute the ln-likelihood of the current model.
 
         """
-        return -0.5 * self.chi2()
+        lnlike = 0.0
 
-    def chi2(self):
-        assert self._data is not None
-        model = self.lightcurve(self._data[0])
-        delta = self._data[1] - model
+        for ds in self.datasets:
+            # Add in the jitter.
+            ivar = ds.ivar
+            inds = ivar > 0
+            ivar[inds] = 1. / (1. / ivar[inds] + ds.jitter * ds.jitter)
 
-        # Add in the jitter.
-        ivar = np.array(self._data[2])
-        # inds = ivar > 0
-        # ivar[inds] = 1. / (1. / ivar[inds] + self.jitter)
+            if ds.__type__ == "lc":
+                # Compute the "foreground" model probability.
+                model = self.lightcurve(ds.time, texp=ds.texp)
+                delta = ds.flux - ds.zp * model
+                e1 = np.log(1 - self.pbad) + 0.5 * np.log(ivar) \
+                     - 0.5 * delta * delta * ivar
 
-        return np.sum(delta * delta * ivar) - np.sum(np.log(ivar))
+                # Compute the background model probability.
+                ivar_bg = np.zeros_like(ivar)
+                ivar_bg[inds] = 1. / (1. / ds.ivar[inds]
+                                      + self.vbad * self.vbad)
+                delta_bg = ds.flux - ds.zp
+                e2 = np.log(self.pbad) + 0.5 * np.log(ivar_bg) \
+                     - 0.5 * delta_bg * delta_bg * ivar_bg
 
-    def lightcurve(self, t):
+                lnlike += np.sum(np.logaddexp(e1, e2))
+
+            elif ds.__type__ == "rv":
+                model = self.radial_velocity(ds.time)
+                delta = ds.rv - model
+                c2 = np.sum(delta * delta * ivar) - np.sum(np.log(ivar))
+                lnlike -= 0.5 * c2
+
+            else:
+                raise TypeError()
+
+        return lnlike
+
+    def _get_pars(self):
+        r = [(p.mass, p.r, p.a, p.t0, p.e, p.pomega, p.ix, p.iy)
+                                                    for p in self.planets]
+        return zip(*r)
+
+    def lightcurve(self, t, texp=54.2, K=3):
         """
         Get the light curve of the model at the current model.
 
         :param t:
             The times where the light curve should be evaluated.
 
+        :param texp:
+            The exposure time in seconds.
+
+        :param K:
+            The number of bins to use when integrating over exposure time.
+
         """
+        mass, r, a, t0, e, pomega, ix, iy = self._get_pars()
         s = self.star
-        r = [(p.r, p.a, p.t0, p.e, p.pomega, p.ix, p.iy) for p in self.planets]
-        r, a, t0, e, pomega, ix, iy = zip(*r)
-        ldp = self.star.ldp
-        lc, info = _bart.lightcurve(t, s.flux, s.mass, s.radius, self.iobs,
-                                r, a, t0, e, pomega, ix, iy,
+        ldp = s.ldp
+        lc, info = _bart.lightcurve(t, texp / 68400., K, s.flux, s.mass,
+                                s.radius, self.iobs,
+                                mass, r, a, t0, e, pomega, ix, iy,
                                 ldp.bins, ldp.intensity)
         assert info == 0, "Orbit computation failed. {0}".format(e)
         return lc
 
-    def _prepare_data(self, t, f, ferr):
-        """
-        Sanitize some light curve data.
+    def radial_velocity(self, t):
+        s = self.star
+        result = np.zeros_like(t)
+        for p in self.planets:
+            pos, rv, info = _bart.solve_orbit(t, s.mass, p.mass, p.e, p.a,
+                                            p.t0, p.pomega,
+                                            np.radians(90 - self.iobs + p.ix),
+                                            np.radians(p.iy), 1)
+            result += rv
 
-        """
-        # Deal with masked and problematic data points.
-        inds = ~(np.isnan(t) + np.isnan(f) + np.isnan(ferr)
-            + np.isinf(t) + np.isinf(f) + np.isinf(ferr)
-            + (f < 0) + (ferr <= 0))
-        t, f, ivar = t[inds], f[inds], 1.0 / ferr[inds] / ferr[inds]
+        # Add systemic velocity and convert to m/s.
+        return self.rv0 + 8050.0 * result
 
-        # Store the data.
-        self._data = [t, f, ivar]
+    def optimize(self):
+        objective = lambda p: -self(p)
+        result = op.minimize(objective, self.vector)
+        self.vector = result.x
+        return result.x
 
-    def fit(self, data, iterations, start=None, filename="./mcmc.h5",
-            **kwargs):
+    def fit(self, iterations, start=None, filename="mcmc.h5", **kwargs):
         """
         Fit the data using MCMC to get constraints on the parameters.
 
-        :param data:
-            A tuple of the form ``(t, f, ferr)`` giving the data to fit.
-
         :param iterations:
             The number of MCMC steps to run in the production pass.
 
@@ -378,6 +507,8 @@ class PlanetarySystem(Model):
             The number of clusters to use for K-means in the trimming step.
 
         """
+        outfn = os.path.join(self.basepath, filename)
+
         # Reset for the sake of pickling.
         self._sampler = None
 
@@ -403,43 +534,40 @@ class PlanetarySystem(Model):
 
         # Parse the other input parameters.
         threads = kwargs.get("threads", 10)
-        burnin = kwargs.get("burnin", [300, ])
+        burnin = kwargs.get("burnin", [])
         K = kwargs.get("K", 4)
         thin = kwargs.get("thin", 1)
 
-        # Sanitize the data.
-        self._prepare_data(*data)
-
         # Initialize a sampler.
         s = emcee.EnsembleSampler(nwalkers, ndim, self, threads=threads)
 
         # Do some HACKISH initialization. Start with a small ball and then
         # iterate (shrinking the size of the ball each time) until the range
         # of log-probabilities is "acceptable".
-        print("Initializing walkers.")
-        ball = 1e-5
-        p0 = self.sample(nwalkers, std=ball)
-        lp = s._get_lnprob(p0)[0]
-        dlp = np.var(lp)
-        while dlp > 2:
-            ball *= 0.5
+        if start is None:
+            print("Initializing walkers.")
+            ball = 1e-5
             p0 = self.sample(nwalkers, std=ball)
             lp = s._get_lnprob(p0)[0]
             dlp = np.var(lp)
+            while dlp > 2:
+                ball *= 0.5
+                p0 = self.sample(nwalkers, std=ball)
+                lp = s._get_lnprob(p0)[0]
+                dlp = np.var(lp)
+        else:
+            p0 = np.array(start)
 
         # Run the burn-in iterations. After each burn-in run, cluster the
         # walkers and discard the worst ones.
         for i, nburn in enumerate(burnin):
             print(u"Burn-in pass {0}...".format(i + 1))
-            p0, lprob, state = s.run_mcmc(p0, nburn, storechain=False)
+            p0, lprob, state = s.run_mcmc(p0, nburn)
 
-            # Cluster the positions of the walkers at their final position
-            # in log-probability using K-means.
-            mix = mog.MixtureModel(K, np.atleast_2d(lprob).T)
-            mix.run_kmeans()
-
-            # Extract the cluster memberships.
-            rs, rmxs = mix.kmeans_rs, np.argsort(mix.means.flatten())
+            lp = s.lnprobability
+            mu, rs = km1d(lp.reshape(lp.size), k=K)
+            rs = rs.reshape(lp.shape)[:, -1]
+            rmxs = np.argsort(mu)
 
             # Determine the "best" cluster that actually has walkers in it.
             for rmx in rmxs[::-1]:
@@ -479,6 +607,8 @@ class PlanetarySystem(Model):
         pars = self.parameters + self.star.parameters
         for p in self.planets:
             pars += p.parameters
+        for d in self.datasets:
+            pars += d.parameters
         par_list = np.array([str(pickle.dumps(p, 0)) for p in pars])
 
         # Pickle the initial conditions of ``self`` so that we can start again
@@ -489,13 +619,10 @@ class PlanetarySystem(Model):
         from bart import __version__
 
         # Initialize the results file.
-        with h5py.File(filename, u"w") as f:
+        with h5py.File(outfn, u"w") as f:
             # Keep track of the current Bart version.
             f.attrs["version"] = __version__
 
-            # Save the dataset to the file.
-            f.create_dataset("data", data=np.vstack(self._data))
-
             # Save the list of parameters and their pickled representations
             # to the file.
             f.create_dataset("parlist", data=par_list)
@@ -503,6 +630,9 @@ class PlanetarySystem(Model):
             # Save the pickled version of ``self``.
             f.create_dataset("initial_pickle", data=initial_spec)
 
+            f.create_dataset("datasets",
+                             data=str(pickle.dumps(self.datasets)))
+
             # Add a group and headers for the MCMC results.
             g = f.create_group("mcmc")
 
@@ -530,7 +660,7 @@ class PlanetarySystem(Model):
                                                     iterations=iterations)):
             if i % thin == 0:
                 print(status_fmt.format(i, np.mean(s.acceptance_fraction)))
-                with h5py.File(filename, "a") as f:
+                with h5py.File(outfn, "a") as f:
                     g = f["mcmc"]
                     c_ds = g["chain"]
                     lp_ds = g["lnprob"]
@@ -555,15 +685,3 @@ class PlanetarySystem(Model):
 
         self._sampler = s
         return self._sampler.flatchain
-
-
-# class LDPParameter(LogParameter):
-
-#     def getter(self, ps):
-#         return self.conv(ps.ldp.intensity[self.ind]
-#                          - ps.ldp.intensity[self.ind + 1])
-
-#     def setter(self, ps, val):
-#         j = self.ind
-#         ps.ldp.intensity.__setitem__(j + 1,
-#                                      ps.ldp.intensity[j] - self.iconv(val))
