@@ -1,6 +1,6 @@
-      subroutine lightcurve(n, t, &
+      subroutine lightcurve(n, t, texp, nbin, &
                             fstar, mstar, rstar, iobs, &
-                            np, r, a, t0, e, pomega, ix, iy, &
+                            np, mass, r, a, t0, e, pomega, ix, iy, &
                             nld, rld, ild, &
                             flux, info)
 
@@ -12,6 +12,12 @@
         ! :param t: (double precision(n))
         !   The times where the lightcurve should be evaluated in days.
         !
+        ! :param texp: (double precision)
+        !   The exposure time in days.
+        !
+        ! :param nbin: (integer)
+        !   The number of bins to integrate for an exposure time.
+        !
         ! :param fstar: (double precision)
         !   The un-occulted flux of the star in arbitrary units.
         !
@@ -28,15 +34,17 @@
         ! :param np: (integer)
         !   The number of planets in the system.
         !
+        ! :param mass: (double precision(np))
+        !   The masses of the planets in Solar masses.
+        !
         ! :param r: (double precision(np))
-        !   The sizes of the planets in units of the star's radius.
+        !   The sizes of the planets in Solar radii.
         !
         ! :param a: (double precision(np))
-        !   The semi-major axes of the orbits in units of the star's
-        !   radius.
+        !   The semi-major axes of the orbits in Solar radii.
         !
         ! :param t0: (double precision(np))
-        !   The time of a reference pericenter passage in days.
+        !   The time of a reference transit in days.
         !
         ! :param e: (double precision(np))
         !   The eccentricities of the orbits.
@@ -69,6 +77,10 @@
         ! :returns flux: (double precision(n))
         !   The observed flux at each time ``t`` in the same units as
         !   the input ``fstar``.
+        !
+        ! :returns info: (integer)
+        !   Was the computation successful? If ``info==0``, it was.
+        !   Otherwise, no solution was found for Kepler's equation.
 
         implicit none
 
@@ -78,13 +90,17 @@
         integer, intent(in) :: n
         double precision, dimension(n), intent(in) :: t
 
+        ! Exposure time integration.
+        double precision, intent(in) :: texp
+        integer, intent(in) :: nbin
+
         ! The properties of the star and the system.
         double precision, intent(in) :: fstar,mstar,rstar,iobs
 
         ! The planets.
         integer, intent(in) :: np
         double precision, dimension(np), intent(in) :: &
-                                          r, a, e, t0, pomega, ix, iy
+                                      mass, r, a, e, t0, pomega, ix, iy
 
         ! The limb-darkening profile.
         integer, intent(in) :: nld
@@ -97,23 +113,33 @@
         integer, intent(out) :: info
 
         integer :: i, j
-        double precision, dimension(3, n) :: pos
-        double precision, dimension(n) :: b, tmp
+        double precision, dimension(3, n*nbin) :: pos
+        double precision, dimension(n*nbin) :: b, tmp, ttmp, ftmp, rvtmp
 
         info = 0
 
-        ! Initialize the full lightcurve to the un-occulted stellar
+        ! Add extra time slices for integration over exposure time.
+        do i=1,n
+          do j=1,nbin
+
+            ttmp((i-1)*nbin+j) = t(i) + texp * ((j-0.5)/float(nbin) - 0.5)
+
+          enddo
+        enddo
+
+        ! Initialize the full light curve to the un-occulted stellar
         ! flux.
-        flux(:) = fstar
+        ftmp(:) = fstar
 
         ! Loop over the planets and solve for their orbits and transit
         ! profiles.
         do i=1, np
 
-          call solve_orbit(n, t, mstar, &
-                           e(i), a(i) * rstar, t0(i), pomega(i), &
-                           (90.d0 - iobs + ix(i)) / 180.d0 * pi, iy, &
-                           pos, info)
+          call solve_orbit(n*nbin, ttmp, mstar, mass(i), &
+                           e(i), a(i), t0(i), pomega(i), &
+                           (90.d0 - iobs + ix(i)) / 180.d0 * pi, &
+                           iy / 180.d0 * pi, &
+                           0, pos, rvtmp, info)
 
           ! Make sure that the orbit was properly solved.
           if (info.ne.0) then
@@ -123,15 +149,24 @@
           b = dsqrt(pos(2,:) * pos(2,:) + pos(3,:) * pos(3,:)) / rstar
 
           ! HACK: deal with positions behind star.
-          do j=1, n
+          do j=1,n*nbin
             if (pos(1,j) .le. 0.0d0) then
               b(j) = 1.1d0 + r(i)
             endif
           enddo
 
-          call ldlc(r(i), nld, rld, ild, n, b, tmp)
-          flux = flux * tmp
+          call ldlc(r(i) / rstar, nld, rld, ild, n*nbin, b, tmp)
+
+          ftmp = ftmp * tmp
 
         enddo
 
+        ! "Integrate" over exposure time.
+        flux(:) = 0
+        do i=1,n
+          do j=1,nbin
+            flux(i) = flux(i) + ftmp((i-1)*nbin + j) / float(nbin)
+          enddo
+        enddo
+
       end subroutine
